@@ -27,11 +27,12 @@ void
 handle_slave_connection (int input, int output)
 {
 	glibtop *server G_GNUC_UNUSED = glibtop_global_server;
-	gint64 *param_ptr G_GNUC_UNUSED;
 	const void *ptr G_GNUC_UNUSED;
 
 	unsigned short max_len G_GNUC_UNUSED;
 	pid_t pid G_GNUC_UNUSED;
+	gint64 proc_which G_GNUC_UNUSED;
+	gint64 proc_arg G_GNUC_UNUSED;
 
 	glibtop_response _resp, *resp = &_resp;
 	glibtop_command _cmnd, *cmnd = &_cmnd;
@@ -40,13 +41,11 @@ handle_slave_connection (int input, int output)
 	glibtop_send_version (glibtop_global_server, output);
 
 	while (do_read (input, cmnd, sizeof (glibtop_command))) {
-#ifdef SLAVE_DEBUG
-		fprintf (stderr, "Slave %d received command "
-			 "%llu from client.\n", getpid (), cmnd->command);
-#endif
+		glibtop_debug ("Slave %d received command "
+			 "%lu from client.", getpid (), cmnd->command);
 
 		if (cmnd->data_size >= BUFSIZ)
-			glibtop_error ("Client sent %llu bytes, "
+			glibtop_error ("Client sent %lu bytes, "
 				       "but buffer is %lu",
 				       cmnd->size, (unsigned long)BUFSIZ);
 
@@ -55,10 +54,8 @@ handle_slave_connection (int input, int output)
 		memset (parameter, 0, sizeof (parameter));
 
 		if (cmnd->data_size) {
-#ifdef SLAVE_DEBUG
-			fprintf (stderr, "Client has %llu bytes of data.\n",
+			glibtop_debug ("Client has %lu bytes of data.",
 				 cmnd->data_size);
-#endif
 
 			do_read (input, parameter, cmnd->data_size);
 
@@ -72,10 +69,11 @@ handle_slave_connection (int input, int output)
 			return;
 #if GLIBTOP_SUID_PROCLIST
 		case GLIBTOP_CMND_PROCLIST:
-			param_ptr = (gint64 *) parameter;
+			memcpy(&proc_which, parameter, sizeof proc_which);
+			memcpy(&proc_arg, parameter + sizeof proc_which, sizeof proc_arg);
 			ptr = glibtop_get_proclist_p
 				(server, &resp->u.data.proclist,
-				 param_ptr [0], param_ptr [1]);
+				 proc_which, proc_arg);
 			do_output (output, resp, _offset_data (proclist),
 				   resp->u.data.proclist.total, ptr);
 			g_free (ptr);
@@ -137,6 +135,12 @@ handle_slave_command (glibtop_command *cmnd, glibtop_response *resp,
 		resp->offset = _offset_data (cpu);
 		break;
 #endif
+#if GLIBTOP_SUID_DISK
+	case GLIBTOP_CMND_DISK:
+		glibtop_get_disk_p (server, &resp->u.data.disk);
+		resp->offset = _offset_data (disk);
+		break;
+#endif
 #if GLIBTOP_SUID_MEM
 	case GLIBTOP_CMND_MEM:
 		glibtop_get_mem_p (server, &resp->u.data.mem);
@@ -211,6 +215,14 @@ handle_slave_command (glibtop_command *cmnd, glibtop_response *resp,
 		resp->offset = _offset_data (proc_time);
 		break;
 #endif
+#if GLIBTOP_SUID_PROC_IO
+	case GLIBTOP_CMND_PROC_IO:
+		memcpy (&pid, parameter, sizeof (pid_t));
+		glibtop_get_proc_io_p
+			(server, &resp->u.data.proc_io, pid);
+		resp->offset = _offset_data (proc_time);
+		break;
+#endif
 #if GLIBTOP_SUID_PROC_SIGNAL
 	case GLIBTOP_CMND_PROC_SIGNAL:
 		memcpy (&pid, parameter, sizeof (pid_t));
@@ -249,7 +261,7 @@ handle_slave_command (glibtop_command *cmnd, glibtop_response *resp,
 		break;
 #endif
 	default:
-		glibtop_error ("Child received unknown command %llu",
+		glibtop_error ("Child received unknown command %lu",
 			       cmnd->command);
 		break;
 	}
